@@ -7,6 +7,8 @@ from torch.utils.data import DataLoader, Dataset
 
 logger = logging.getLogger(__name__)
 
+OOD_STRING = '__out-of-distribution__'
+
 
 def encode_sentence(sentence, max_length, tokenizer):
     input_question = tokenizer.encode_plus(sentence, add_special_tokens=True,
@@ -18,15 +20,19 @@ def encode_sentence(sentence, max_length, tokenizer):
             'tt': input_question['token_type_ids'].squeeze(0)}
 
 
-def get_passages_by_source(json_data):
+def get_passages_by_source(json_data, keep_ood=True):
     source2passages = defaultdict(list)
     pid2passage = {}
     pid2index = {}
     for passage in json_data['passages']:
+
+        if not is_in_distribution(passage) and not keep_ood:
+            continue  # not keeping OOD
+
         source = passage['source']
         passage_id = passage['passage_id']
-
         source2passages[source].append(passage)
+
         if passage_id in pid2passage:
             raise ValueError('duplicate passage id: {}'.format(passage_id))
 
@@ -45,7 +51,7 @@ def is_in_distribution(passage):
     return reference_type.lower().startswith('faq')
 
 
-def _encode_passages(source2passages, max_passage_length, tokenizer):
+def _encode_passages(source2passages, max_passage_length, tokenizer, do_not_encode=False):
     """
     note - this will only encode in-distribution passages.
     :param source2passages:
@@ -60,8 +66,11 @@ def _encode_passages(source2passages, max_passage_length, tokenizer):
         for passage in passages:
             if is_in_distribution(passage):
                 passage_text = get_passage_last_header(passage)
-                encoded_passage = encode_sentence(passage_text, max_passage_length, tokenizer)
-                source2encoded_passages[source].append(encoded_passage)
+                if do_not_encode:
+                    source2encoded_passages[source].append(passage_text)
+                else:
+                    encoded_passage = encode_sentence(passage_text, max_passage_length, tokenizer)
+                    source2encoded_passages[source].append(encoded_passage)
                 source2id[source] += 1
             else:
                 source2ood[source] += 1
@@ -74,7 +83,7 @@ def get_passage_last_header(passage, return_error_for_ood=False):
     elif return_error_for_ood:
         raise ValueError('passage is ood')
     else:
-        return '__out-of-distribution__'
+        return OOD_STRING
 
 
 def get_question(example):
@@ -85,9 +94,21 @@ def get_passage_id(example):
     return example['passage_id']
 
 
+def get_examples(json_data, keep_ood):
+    examples = []
+    # always keep ood here, because we need it for the ood check later on
+    _, pid2passage, _ = get_passages_by_source(json_data, keep_ood=True)
+    for example in json_data['examples']:
+        related_passage_id = get_passage_id(example)
+        related_passage = pid2passage[related_passage_id]
+        if is_in_distribution(related_passage) or keep_ood:
+            examples.append(example)
+    return examples
+
+
 class ReRankerDataset(Dataset):
 
-    def __init__(self, json_file, max_example_len, max_passage_len, tokenizer):
+    def __init__(self, json_file, max_example_len, max_passage_len, tokenizer, keep_ood):
         self.max_example_len = max_example_len
         self.max_passage_len = max_passage_len
         self.tokenizer = tokenizer
@@ -99,18 +120,20 @@ class ReRankerDataset(Dataset):
             json_data = json.load(in_stream)
 
         source2passages, pid2passage, pid2index = get_passages_by_source(
-            json_data)
+            json_data, keep_ood=keep_ood)
 
         self.encoded_source2passages, source2id, source2ood = _encode_passages(
             source2passages, max_passage_len, tokenizer)
         self.pid2passage = pid2passage
         self.pid2index = pid2index
-        self.examples = json_data['examples']
+        self.examples = get_examples(json_data, keep_ood=keep_ood)
         logger.info('loaded passages from file {} - found {} sources'.format(
             json_file, len(source2id)))
         for source in source2id.keys():
             logger.info('source "{}": found {} in-distribution and {} out-of-distribution'.format(
                 source, source2id[source], source2ood[source]))
+        logger.info('keeping OOD? {}'.format(keep_ood))
+        self.keep_ood = keep_ood
 
     def __len__(self):
         return len(self.examples)
@@ -126,13 +149,16 @@ class ReRankerDataset(Dataset):
         if is_in_distribution(passage):
             target_idx = self.pid2index[passage_id]
         else:
-            target_idx = -1
+            if self.keep_ood:
+                target_idx = -1
+            else:
+                raise ValueError('found ood - but keep_ood has been used..')
         source = self.pid2passage[passage_id]['source']
         return {'question': encoded_question, 'target_idx': target_idx,
                 'passages': self.encoded_source2passages[source]}
 
 
 def generate_dataloader(data_file, max_question_len, max_paragraph_len, tokenizer, batch_size,
-                        num_workers, shuffle):
-    dataset = ReRankerDataset(data_file, max_question_len, max_paragraph_len, tokenizer)
+                        num_workers, shuffle, keep_ood):
+    dataset = ReRankerDataset(data_file, max_question_len, max_paragraph_len, tokenizer, keep_ood)
     return DataLoader(dataset, batch_size=batch_size, num_workers=num_workers, shuffle=shuffle)
